import type { Pupil } from 'medama';
import { useContext, useMemo } from 'react';
import { pupilProviderContext, type PupilProviderProps } from './ProvideMedama';

export type MedamaReactHookOptions<State extends object> =
  | { pupil: Pupil<State>; id?: undefined }
  | { id: string | number | symbol; pupil?: undefined };

export const useMedama = <State extends object>({
  pupil: pupilIfProvided,
  id,
}: Partial<MedamaReactHookOptions<State>> = {}): Pupil<State> => {
  const contextValue = useContext(pupilProviderContext) as PupilProviderProps<State>;

  const pupil = useMemo(() => {
    if (pupilIfProvided) return pupilIfProvided;

    const { pupil: pupilFromContext } =
      id == null || id === contextValue.id ? contextValue : (contextValue.lookUp?.(id) ?? {});

    return pupilFromContext;
  }, [pupilIfProvided, id, contextValue]);

  if (pupil == null)
    throw new Error(
      'Medama Error: ProvideMedama component must be initialized up in component tree'
    );

  return pupil;
};
